#include "DPE_API.h"

#include <stdio.h> 
#include <stdlib.h> 
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <math.h>
#include <algorithm>
#include <sstream>

using namespace std;

template <class T> void PrintVec(std::vector<T> v_Vec)
{
	for (unsigned int i = 0; i < v_Vec.size(); ++i)
	{
		cout << v_Vec[i] << "|";
	}
	cout << "\n";
	return;
}

template <class T> void PrintVecVec(std::vector<std::vector<T> > vv_Vec)
{
	for (unsigned int i = 0; i < vv_Vec.size(); ++i)
	{
		PrintVec(vv_Vec[i]);
	}
	return;
}

int GetPrintResults()
{
	int RetCon = DPE_ERR_NO_ERR; 

	std::vector<std::vector<double> > DoseTimeClass;
	RetCon = DPE_GetDoseTimeClass(DoseTimeClass);
	std::vector<std::vector<double> > FluxTimeClass;
	RetCon = DPE_GetFluxTimeClass(FluxTimeClass);

	printf("\nDose - sampling, class\n");
	PrintVecVec(DoseTimeClass);
	printf("\nFlux - sampling, class\n");	
	PrintVecVec(FluxTimeClass);
	printf("\n");

	double EnergyDep = 0;
	DPE_GetEnergyDep(EnergyDep);
	double DoseRate = 0;
	DPE_GetDoseRate(DoseRate);
	
	printf("EnergyDep\t%f\n", EnergyDep );
	printf("DoseRate\t%f\n", DoseRate );		

	printf("\n");

	double EnergyDepCurr = 0;
	DPE_GetCurrentEnergyDep(EnergyDepCurr);
	double DoseRateCurr = 0;
	DPE_GetCurrentDoseRate(DoseRateCurr, 5);
	double Flux = 0;
	DPE_GetCurrentFlux(Flux, 5);
	
	printf("EnergyDepCurr\t%f\n", EnergyDepCurr );
	printf("DoseRateCurr\t%f\n", DoseRateCurr );			
	printf("Flux\t\t%f\n", Flux );			

	printf("\n");

	std::vector<double> EnergyDepCurrClass;
	DPE_GetCurrentEnergyDepClass(EnergyDepCurrClass);
	std::vector<double> DoseRateCurrClass;
	DPE_GetCurrentDoseRateClass(DoseRateCurrClass, 5);
	std::vector<double> FluxClass;
	DPE_GetCurrentFluxClass(FluxClass, 5);
	
	printf("EnergyDepCurr\t\t"); PrintVec(EnergyDepCurrClass);
	printf("DoseRateCurrClass\t"); PrintVec(DoseRateCurrClass);
	printf("FluxClass\t\t"); PrintVec(FluxClass);
	
	printf("\n-----\n\n");

	return RetCon;
}

int Example_ProcessClusters()
{
	int RetCon = DPE_ERR_NO_ERR;

	string FileParam_Path = "./";
	string FileParam_Name = "ParametersFile.txt";

	DPE_Construct();

	RetCon = DPE_LoadParamFile(FileParam_Path, FileParam_Name);
	
	RetCon = DPE_SetTimeSampling(1);
	// RetCon = DPE_SetTimeAcquisition(5);

	RetCon = DPE_Init();


	//1st PROCESSING

		std::vector<std::vector<double> > Clusters;
		Clusters.push_back(std::vector<double>{0,0,13.8994,11.7735,384.244,207554.688,0,20,72.807,14,0.718563,1.12138,0.502295,5.80682,3.60281});
		Clusters.push_back(std::vector<double>{0,1,5.8536,11.2997,198.141,244553.125,0,12,39.4045,8,1.17381,1.35694,0.0947886,3.14389,3.14389});
		Clusters.push_back(std::vector<double>{0,2,8.525,9.54095,8.56641,1334110.938,0,2,4.52464,2,1.51062,1.5708,1,1,6.12323e-17});
		Clusters.push_back(std::vector<double>{0,3,4.23094,12.9942,255.264,1761698.438,0,17,40.6845,10,1.02964,0.0352724,0.0195387,4.03278,4.03278});
		Clusters.push_back(std::vector<double>{0,4,4.23094,12.9942,255.264,5000761698.438,0,17,40.6845,10,1.02964,0.0352724,0.0195387,4.03278,4.03278});

		RetCon = DPE_ProcessClusters(Clusters);

		string RadFieldName = "";
		double RadFieldProb = 0;
		RetCon = DPE_RadFieldRecognition(RadFieldName, RadFieldProb);

		printf("Radiation field name\t%s\n", RadFieldName.c_str() );
		printf("Radiation field prob\t%f\n", RadFieldProb );

	//GET AND PRINT RESULTS

		GetPrintResults();

	//2ND PROCESSING

		// RetCon = DPE_Reset();
		RetCon = DPE_ResetCurrDataStorage();	//This is needed if curr date should be only for newly processed clusters.

		Clusters.clear();

		Clusters.push_back(std::vector<double>{0,5,4.23094,12.9942,255.264,5100761698.438,0,17,40.6845,10,1.02964,0.0352724,0.0195387,4.03278,4.03278});
		Clusters.push_back(std::vector<double>{0,6,4.23094,12.9942,200.264,10000761698.438,0,17,40.6845,10,1.02964,0.0352724,0.0195387,4.03278,4.03278});

		RetCon = DPE_ProcessClusters(Clusters);

		RetCon = DPE_RadFieldRecognition(RadFieldName, RadFieldProb);

		printf("Radiation field name\t%s\n", RadFieldName.c_str() );
		printf("Radiation field prob\t%f\n", RadFieldProb );

	//GET AND PRINT RESULTS

		GetPrintResults();

	DPE_Destruct();

	return RetCon;
}


int main(int argc, char const *argv[])
{
	int RetCon = DPE_ERR_NO_ERR;
	int Case = 0;
	
	//LOAD CASE NUM

		if(argc >= 2)
		{
			std::stringstream s(argv[1] );
			s >> Case;
		}

	switch(Case) 
	{
	    case 0 : 
	    {
			RetCon = Example_ProcessClusters();
	    }
	    break;
	    
	    case 1 : 
	    {
			;
	    }
	    break;	    
	}

	return RetCon;
}